//This file is "cal_conc_cor.cc"
//Definition of a member function of class Lattice.
//Calculates the concentration of atomic species and also 
//accumulates the no. of instances for each atomic species
//at each site. The latter is for finding LRO.
//Accumulates terms for pair correlation as well.
//By Teck Leong Tan

#include "lattice.h"

int convert(int noccu1, int noccu2, int num_type);//Return correct species index for <P(x,i), P(y,j)>, i<j

void Lattice:: cal_conc_cor()
{
  C_struct.conc.assign((num_type+1),0.0);//num_type+1 to include zeroth atom type
  C_struct.amount.assign((num_type+1),0);//amount= conc * natoms 
  
  int tempd=0;
  //Loop over cell
  for (int ijk=0; ijk<nsize;ijk++)
  {
    //Loop over no. of sublattices
    for (int l=0; l<num_sublat;l++)
    {
      if (pair_option>0)//Pair correlation
      {
        //For no. of pairs in pair correlation list
        for (int nei=0;nei< noccupancy[ijk].site[l].cor_pair.size();nei++)
        {
          int ijk2=noccupancy[ijk].site[l].cor_pair[nei]->ijk;
          int ll2=noccupancy[ijk].site[l].cor_pair[nei]->l;
          int noccu1,noccu2;
          //Only if noccu1 < noccu2 will terms be accumulated
          if (noccupancy[ijk].site[l].atomtype<noccupancy[ijk2].site[ll2].atomtype)
          {
            noccu1=noccupancy[ijk].site[l].atomtype;
            noccu2=noccupancy[ijk2].site[ll2].atomtype;

            //Count no. of instances for each <P(x,i),P(y,j)> term
            noccupancy[0].site[l].cor_pair[nei]->num_instances[convert(noccu1,noccu2,num_type)]+=1;
            int temp=convert(noccu1,noccu2,num_type);

          }
        }
      }//end if (pair_option>0)

      //Loop over all atomic species
      for (int type=0;type<=num_type;type++)
      { 
                
        if (noccupancy[ijk].site[l].atomtype==type)
        {
          C_struct.amount[type]+=1;

          if (pair_option>0)
          {noccupancy[0].site[l].sub_instances[type]+=1;}

          if (LRO_option==1)
          {noccupancy[ijk].site[l].LRO_instances[type]+=1;}
          
        }
        
      }
    }//end for (int ijk=0; ijk<nsize;ijk++)
  }//end for (int l=0; l<num_sublat;l++)  

  /*for (int i=1;i<=num_type;i++)
  {
    tempd+=C_struct.amount.at(i);
  }
  C_struct.amount[0]=natoms-tempd;*/
  for (int i=0;i<=num_type;i++)
  {
    
    C_struct.conc[i]=(double)C_struct.amount[i]/(double)natoms;
  }
  for (int i=0;i<=num_type;i++)
  {
   //cout<<"conc["<<i<<"]= "<<C_struct.conc[i]<<endl;
  }
  
}

//Convert to correct <P(x,i),P(y,j)> term index
int convert(int noccu1, int noccu2, int num_type)
{ int index=0;
  for (int i=0;i<=noccu1;i++)
  {
    index +=(i>0)*(num_type-i+1);
  }
  index+=(noccu2-noccu1);
  return (index-1);
}

//EOF




