/* This file is create_corn.cc
   Function defintion of create_corn which create a 
   neighbor list for correlations.
   By Teck Leong Tan
*/

#include "lattice.h"
#include "power.h"

void Lattice::create_corn()
{
  int cartesian[ndim];//of cell of interest
  int cartesian2[ndim];//of neighbor cell
  int cartesian_r[ndim];//relative cartesian bet cell of interest and neighbor
  int ijk2;//ijk of neighbor

  //If option is SRO, pair====================================================================
  if (pair_option==2)//If user request all pairs
  { 
    //Loop over sublattice
    for (int l=0;l<num_sublat;l++)
    {
      noccupancy[0].site[l].sub_instances.assign(num_type+1,0);
    }
    
    //Loop over all cells
    for ( int ijk_r=0;ijk_r<nsize;ijk_r++)
    {
      //Convert ijk_r to relative cartesian coordinates 
      ijk_to_cartesian(ijk_r, cartesian_r);
      
      //Loop over num of cells
      for (int ijk=0;ijk<nsize;ijk++)
      {
        //Convert ijk to cartesian coordinates 
        ijk_to_cartesian(ijk, cartesian);
        boundary (cartesian, cartesian2, cartesian_r);
        ijk2=cartesian_to_ijk(cartesian2); 
       
        //Loop over sublattice of point of interest
        for (int ll=0;ll<num_sublat;ll++)
        {
          //Loop over all possible sublattices of neighbor
          for (int ll2=0;ll2<num_sublat;ll2++)
          {
            noccupancy[ijk].site[ll].cor_pair.push_back((new Cor_neighborO));
            noccupancy[ijk].site[ll].cor_pair.back()->ijk=ijk2;
            noccupancy[ijk].site[ll].cor_pair.back()->l=ll2;
            //Initialise num of atoms of given correlation
            for (int i=0;i<num_type;i++)
            {
              for (int j=i+1;j<=num_type;j++)
              {
                noccupancy[ijk].site[ll].cor_pair.back()->num_instances.push_back(0);
              }
            }
          }//end for (int ll2=0;ll2<num_sublat;ll2++)
        }//end for (int ll=0;ll<num_sublat;ll++)
      }// end for (int ijk=0;ijk<nsize;ijk++) 
    }// end for ( int ijk_r=0;ijk_r<nsize;ijk_r++)
  }//end if
  
  else //pair_option==1, i.e user request a cut-off
  {
    //Loop over sublattice
    for (int l=0;l<num_sublat;l++)
    {
      noccupancy[0].site[l].sub_instances.assign(num_type+1,0);
    }

    for ( int ijk_r=0;ijk_r<nsize;ijk_r++)
    {
      double magnitude=0;

      //Convert ijk_r to relative cartesian coordinates 
      ijk_to_cartesian(ijk_r, cartesian_r);
      for (int i=0;i<ndim;i++)
      {
	      magnitude+=(cartesian_r[i]*cartesian_r[i]);
      }
      magnitude=sqrt(magnitude);
      if (magnitude-pair_cutoff<0.01)
      {
	      //Loop of num of cells
	      for (int ijk=0;ijk<nsize;ijk++)
	      {
          //Convert ijk to cartesian coordinates 
          ijk_to_cartesian(ijk, cartesian);
          boundary (cartesian, cartesian2, cartesian_r);//check boundary conditions
          ijk2=cartesian_to_ijk(cartesian2); 

          //Loop over sublattice of point of interest
          for (int ll=0;ll<num_sublat;ll++)
          {
            //Loop over all possible sublattices of neighbor
            for (int ll2=0;ll2<num_sublat;ll2++)
            {
              noccupancy[ijk].site[ll].cor_pair.push_back((new Cor_neighborO));
              noccupancy[ijk].site[ll].cor_pair.back()->ijk=ijk2;
              noccupancy[ijk].site[ll].cor_pair.back()->l=ll2;
              //Initialise num of atoms of given correlation
              for (int i=0;i<num_type;i++)
              {
		            for (int j=i+1;j<=num_type;j++)
                {
                  noccupancy[ijk].site[ll].cor_pair.back()->num_instances.push_back(0);
                }
              }
            }//end for (int ll2=0;ll2<num_sublat;ll2++)
          }//end for (int ll=0;ll<num_sublat;ll++)
        }// end for (int ijk=0;ijk<nsize;ijk++) 
      }
    }// end for ( int ijk_r=0;ijk_r<nsize;ijk_r++)
  }//end else
  //End of SRO pair option=============================================================

  //If SRO user define option===========================================================
  //Warning: This part of the code is not well tested

  if (user_def_option==1)
  { 
    //Use define spatially degenerate clusters-----------------------------------------
    //For number of user defined spatial degenerate clusters   
    for (int jj=0;jj<user_def_param_spa.size();jj++)
    { int templ;
      templ=user_def_param_spa[jj]->coord[0].sublat;
      //Loop over no. of cells;
      for (int ijk=0; ijk<nsize;ijk++)
      {
	      //int templ;
	      //templ=user_def_param_spa[jj]->coord[0].sublat;

        //1st site of a cluster is the site itself
	      noccupancy[ijk].site[templ].cor_def_spa.push_back((new Cor_def_clusterO));
        noccupancy[ijk].site[templ].cor_def_spa.back()->ijk.push_back(ijk);
        noccupancy[ijk].site[templ].cor_def_spa.back()->l.push_back(templ);
        ijk_to_cartesian(ijk, cartesian); 
 
	      //For number of sites in cluster -1 ; i.e the no. of sites other than 1st site 
        for (int s=1;s<user_def_param_spa[jj]->coord.size();s++)
        {
	        //Convert from cartesian to ijk
          for (int i=0;i<ndim;i++)
          {  
	          cartesian_r[i]=user_def_param_spa[jj]->coord[s].cartesian[i];
            boundary (cartesian, cartesian2, cartesian_r);
	          ijk2=cartesian_to_ijk(cartesian2);
	        }
          //noccupancy[ijk].site[templ].push_back((new Cor_def_neighborO));
          noccupancy[ijk].site[templ].cor_def_spa.back()->ijk.push_back(ijk2);
          noccupancy[ijk].site[templ].cor_def_spa.back()->\
          l.push_back(user_def_param_spa[jj]->coord[s].sublat);
        }//End of creating user defined correlation cluster table
      }         
      /*For spatial option, degenerate sites are averaged over. Hence, we need
        only store the number of instances of the clusters in noccupancy[0].site
        [templ].cor_def_spa[s]->num_instances.
      */
      for (int i=0;i<power(num_type,noccupancy[0].site[templ].cor_def_spa.back()->\
                           ijk.size());i++) 
      {
        noccupancy[0].site[user_def_param_spa[jj]->coord[0].sublat].cor_def_spa.back()->\
        num_instances.push_back(0);
      }
    }//for (int jj=0;jj<user_def_param_spa.size();jj++)
    //End of user define spatial cluster---------------------------------------------------------

    //No spatial average. Each cluster is a fixed entity.----------------------------------------
    //For number of user defined clusters   
    for (int jj=0;jj<user_def_param_fix.size();jj++)
    {
       int templ, ijk;
       templ=user_def_param_fix[jj]->coord[0].sublat;  

       //1st site of a cluster is the site itself
       for (int i=0;i<ndim;i++)
       {
         cartesian[i]=user_def_param_fix[jj]->coord[0].cartesian[i];//Cartesian coord of 1st site
       }
       ijk=cartesian_to_ijk(cartesian);
       user_def_param_fix[jj]->first_cell=ijk;
       noccupancy[ijk].site[templ].cor_def_fix.push_back((new Cor_def_clusterO));
       noccupancy[ijk].site[templ].cor_def_fix.back()->ijk.push_back(ijk);
       noccupancy[ijk].site[templ].cor_def_fix.back()->l.push_back(templ);

       //For number of sites in cluster -1 ; i.e the no. of sites other than 1st site 
       for (int s=1;s<user_def_param_fix[jj]->coord.size();s++)
       {       
         //Convert from cartesian to ijk
         for (int i=0;i<ndim;i++)
         {       
           cartesian_r[i]=user_def_param_fix[jj]->coord[s].cartesian[i];
           boundary (cartesian, cartesian2, cartesian_r);
           ijk2=cartesian_to_ijk(cartesian2);
         }       
         //noccupancy[ijk].site[templ].push_back((new Cor_def_neighborO));
         noccupancy[ijk].site[templ].cor_def_fix.back()->ijk.push_back(ijk2);
         noccupancy[ijk].site[templ].cor_def_fix.back()->l.\
             push_back(user_def_param_fix[jj]->coord[s].sublat);          
       }//End of creating user defined correlation cluster table

       /*For fix option, degenerate sites are not averaged over. We need
         only store the number of instances of the clusters in noccupancy[ijk].site
         [templ].cor_def_spa[s]->num_instances.
       */        
       for (int i=0;i<power(num_type,noccupancy[ijk].site[templ].cor_def_spa.back()->\
                            ijk.size());i++) 
       {
         noccupancy[ijk].site[user_def_param_fix[jj]->coord[0].sublat].cor_def_spa.back()->\
         num_instances.push_back(0);
       }
    }//end  for (int jj=0;jj<user_def_param_fix.size();jj++)
     //End of user define fixed corr. clusters----------------------------------------------------
  }
  //End of SRO user define option=================================================================


  //LRO option====================================================================================
  if (LRO_option==1)
  {
    //Loop over cells    
    for (int ijk=0;ijk<nsize;ijk++)
    {
      //Loop over sublattice
      for (int l=0;l<num_sublat;l++)
      {
	      noccupancy[ijk].site[l].LRO_instances.assign(num_type+1,0);
      }
    }
  }
  //End of LRO option===============================================================================



  
  fstream cor_nei;
  cor_nei.open(cor_neighborfile, ios::out);
  if (!cor_nei)
  {
    cout<<"File: "<<cor_neighborfile<<" is not opened!"<<endl;
    exit(1);
  }
  cor_nei<<"# Site ijk, site's sublattice, neighbor ijk, neighbor's sublat."<<endl;
  for (int ijk=0;ijk<nsize;ijk++)
  {
    for (int l=0;l<num_sublat;l++)
    {
      for (int nei=0;nei<noccupancy[ijk].site[l].cor_pair.size();nei++)
      {
        cor_nei<<ijk<<" "<<l<<"  ";
        cor_nei<<noccupancy[ijk].site[l].cor_pair[nei]->ijk<<" ";
        cor_nei<<noccupancy[ijk].site[l].cor_pair[nei]->l<<endl;
      }//end for (int nei=0;)
    }//end for (int l=0;)
  }//end for (int ijk=0;)
  cor_nei.close();
  //End of print out to outnei;---------------------------------------
}  
  
   
//EOF
